@echo off
:::::::::::::::::::::::::::::::::::::::::: SETUP ::::::::::::::::::::::::::::::::::::::::::
::
::   1. Install git for windows (https://git-for-windows.github.io/)
::   2. Get UMSBuilder  (http://sourceforge.net/projects/unimediaserver/files/Official%20Releases/Windows/)
::   3. Put this file in your UMSBuilder top folder, double-click it and follow the menus.
::
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

::NOTE: uncomment and edit the line below if automatic git detection fails
::set GIT=c:\path\to\Git

setlocal enabledelayedexpansion

for /d %%d in ("%CD%\JDK*") do set JAVA_HOME=%%d
for /d %%d in ("%CD%\Maven*") do set M2_HOME=%%d
set M2=%M2_HOME%\bin
if not exist "%GIT%" (
	set GIT=
	call :find_msysgit
)
set GITPATH=%GIT%bin;%GIT%usr\bin
echo git: %GITPATH%
set PATH=%GITPATH%;%M2%;%JAVA_HOME%\bin;%PATH%

:getopt
if "%~1"=="" goto no_opts
if /i "%~1"=="compile" (set compile=yes) else set id=%1
shift && goto getopt
:no_opts

set _title=UMSBuilderV2.8
title %_title%

set source=%cd%\UniversalMediaServer
if not exist "%source%" call :clone
cd "%source%"

:: pull
git stash
git checkout master
git pull

if "%compile%" == "yes" goto have_target

:: select
call :menu "Select a target platform" 1 "windows,linux,compile"
set platform=%sel%
if "%platform%" == "windows" (
	set pkg=setup.exe
	set ext=exe
) else if "%platform%" == "linux" (
	set pkg=distribution.tar.gz
	set ext=tgz
) else (
	set compile=yes
)

if exist ~menu.txt del ~menu.txt

:have_target
set sel=%id%
set checkout=%id%
if not "%sel%" == "" goto checkout

set ff=yes
:: white out (),;= and escape &<>!%
set esc=sed -e "s/[(),;=]/ /g" ^| sed -e "s/\([&<>!]\)/^\1/g" ^| sed -e "s/[%%]/%%%%/g"

call :menu "Select an option" 1 "Build the master branch,Choose another branch,Choose a pull request,Show all,Rebuild,Specify a commit or tag"
if %optind% equ 1 goto build
if %optind% equ 2 goto branches
if %optind% equ 3 goto pulls
if %optind% equ 4 goto all
if %optind% equ 5 goto existing
if %optind% equ 6 goto specify

:branches
set n=branch
curl -i "https://api.github.com/repos/UniversalMediaServer/UniversalMediaServer/branches?per_page=1000" 2>nul | grep name | sed -r "s/.+: \"(.+)\",/\1/" | %esc% > ~menu.txt
goto read_menu_txt

:pulls
set n=pull request
curl -i "https://api.github.com/repos/UniversalMediaServer/UniversalMediaServer/pulls?per_page=1000" 2>nul | grep -E "\"number\"|\"title\":" | sed "N; s/\n / /; P; D" | sed -r "s/.+: ([0-9]+),.+: \"(.+)\",/pr\/\1 - \2/" | %esc% > ~menu.txt
goto read_menu_txt

:all
set n=branch
set b=
for /f "tokens=1* delims=/" %%i in ('git branch -r') do set b=!b!,%%j
mode con lines=1000
goto sel

:existing
set n=existing branch
set b=
for /f "tokens=*" %%i in ('git branch') do set b=!b!,%%i
goto sel

:specify
call :get_text "Specify a commit or tag" q
set ff=
set on_err=specify
goto checkout

:read_menu_txt
for /f "tokens=*" %%i in (~menu.txt) do set b=!b!,%%i
del ~menu.txt
goto sel

:sel
call :menu "Select a %n%" q "%b:>=%"
for /f "delims=+ " %%i in ("%sel%") do set sel=%%i

:checkout
git checkout %sel%
if errorlevel 1 call :err failed to checkout %sel% 

:: for some reason 'refs/heads/pr/362' won't pull but 'refs/pull/362/head' will
if not "%sel:pr/=%" == "%sel%" git config "branch.%sel%.merge" "refs/pull/%sel:pr/=%/head" 

if not "%ff%" == "" git pull --ff-only
if errorlevel 1 call :err pull failed


:build
if "%id%" == "" for /f "tokens=1,2,3" %%i in ('git branch -v') do if "%%i" == "*" set id=%%j-%%k
set id=%id:/=%

if not "%compile%" == "yes" goto build_package
title %_title%: compiling ums-%id%
call mvn -DskipTests -Doffline compile
pause
exit

:build_package
title %_title%: building ums-%id%.%ext%

findstr /c:"mvn external:install" pom.xml
if errorlevel 1 (
	call mvn com.savage7.maven.plugins:maven-external-dependency-plugin:resolve-external
	if errorlevel 1 call :err resolve-external failed
	call mvn com.savage7.maven.plugins:maven-external-dependency-plugin:install-external
	if errorlevel 1 call :err install-external failed
) else (
	call mvn external:install
	if errorlevel 1 call :err external:install failed
)

call mvn -P %platform% package
if errorlevel 1 call :err build failed

:: clean up
move /y target\ums*-%pkg% "..\ums-%id%.%ext%"
move /y target\ums.jar "..\ums-%id%.jar"

md keep
move /y target\bin keep
move /y target\maven-status keep
call mvn clean
move /y keep target

:: done
cd ..
echo.
echo Done: ums-%id%.%ext%
echo.
pause
exit /b

:: ------------------------------------- subroutines -------------------------------------

:clone
	git clone https://github.com/UniversalMediaServer/UniversalMediaServer.git
	cd "%source%"
	git config --add remote.origin.fetch "+refs/pull/*/head:refs/remotes/origin/pr/*"
	cd ..
	exit /b

:get_text msg default
	set msg=%~1
	set default=%~2
	set sel=
	cls
	echo ------------- MENU -------------
	echo.
	set /p "sel=%msg% (q=quit) [%default%]: "
	if "%sel%" == "" set sel=%default%
	if "%sel%" == "q" exit 
	exit /b


:menu msg default items
	set msg=%~1
	set default=%~2
	set ct=0
	set items=%~3
	set items=%items: =+%
	cls
	echo ------------- MENU -------------
	for %%i in (%items%) do (
		set /a ct+=1
		set label=%%i
		echo. [!ct!] !label:+= !
	)
	echo.
	:choose
	set optind=
	set /p "index=%msg% (q=quit) [%default%]: "
	if "%index%" == "" (set index=%default%) else if "%index%" == "0" set index=%default%
	if "%index%" == "q" exit 
	if %index% gtr 0 if %index% leq %ct% (
		set optind=%index%
		for %%i in (%items%) do (
			set /a index-=1
			if !index! equ 0 (
				set sel=%%i
				exit /b
			)
		)
	)
	echo invalid item: %index%
	goto :choose

:: from http://stackoverflow.com/questions/8507368/finding-the-path-where-git-is-installed-on-a-windows-system
:find_msysgit
	for %%k in (HKCU HKLM) do (
		for %%w in (\ \Wow6432Node\) do (
			for /f "skip=2 delims=: tokens=1*" %%a in ('reg query "%%k\SOFTWARE%%wMicrosoft\Windows\CurrentVersion\Uninstall\Git_is1" /v InstallLocation 2^> nul') do (
				for /f "tokens=3" %%z in ("%%a") do (
					if exist "%%z:%%b" (
						set GIT=%%z:%%b
						exit /b
					)
				)
			)
		)
	)
	:: last resort
	if "%GIT%" == "" for %%k in ("%ProgramW6432%" "%ProgramFiles%") do (
		echo checking %%k\Git
		if exist "%%k\Git\" (
			set GIT=%%k\Git\
			exit /b
		)
	)
	exit /b

:err
	echo.
	echo Error: %*
	echo.
	pause
	git reset --hard
	git checkout master
	if "%on_err%" == "" exit 1
	goto %on_err%
