#!/bin/bash
########################################## SETUP ##########################################
#
#   1. Install the git and curl packages for your distro, e.g. on ubuntu/debian:
#         sudo apt-get install git curl
#   2. Get UMSBuilder (http://sourceforge.net/projects/unimediaserver/files/Official%20Releases/Linux/)
#   3. Put this file in your UMSBuilder top folder, run it and follow the menus:
#         ./buildv2.sh
#
###########################################################################################

function menu() {
	msg=$1
	default=$2
	echo $@
	eval items=(${@:3})
	len=${#items[@]}
	clear
	echo "------------- MENU -------------"
	for i in $(seq 1 $len) ; do
		echo "[$i] ${items[((i-1))]}"
	done
	echo ""
	while : ; do
		read -p "$msg (q=quit) [$default]: " index
		[ "$index" = "" ] && index=$2
		[ "$index" = "q" ] && exit;
		if [ $index -gt $len ] || [ $index -lt 1 ] ; then echo "invalid choice: $index"; continue; fi
		choice=${items[((index-1))]}
		break
	done
}

function get_text() {
	msg=$1
	default=$2
	choice=
	clear
	echo "------------- MENU -------------"
	echo ""
	read -p "$msg (q=quit) [$default]: " choice
	[ "$choice" = "" ] && choice=$2
	[ "$choice" = "q" ] && exit
	echo ""
}

function branches() {
	items=$(curl -i "https://api.github.com/repos/UniversalMediaServer/UniversalMediaServer/branches?per_page=1000" 2>/dev/null | grep name | sed -r "s/.+: \"(.+)\",/\1/")
	menu "Select a branch" q $items
	branch="$choice"
}

function prs() {
	items=$(curl -i "https://api.github.com/repos/UniversalMediaServer/UniversalMediaServer/pulls?per_page=1000" 2>/dev/null | grep -E "\"number\"|\"title\":" | sed "N; s/\n / /; P; D" | sed -r "s/.+: ([0-9]+),.+: \"(.+)\",/\"pr\/\1 - \2\"/")
	menu "Select a pull request" q $items
	branch="${choice%% *}"
}

function all() {
	items=$(git branch -r | sed "s/origin\///" | sed "s/\\s*\([^\n]*\)/\"\\1\"/g")
	menu "Select a branch" q $items
	branch="$choice"
}

function existing() {
	items=$(git branch | sed "s/\*//g" | sed "s/\\s*\([^\n]*\)/\"\\1\"/g")
	menu "Select an existing branch" q $items
	branch="$choice"
}

function specific() {
	id=
	get_text "Specify a commit or tag" q
	id="$choice"
}


#setup

echo "UMSBuilderV2.8"

set -e
export JAVA_HOME=$(find "$PWD" -name "JDK*"|tail -1)
export M2_HOME=$(find "$PWD" -name "Maven*"|tail -1)
export M2="$M2_HOME"/bin
export PATH="$M2":"$JAVA_HOME"/bin:$PATH

for opt in "$@" ; do
case $opt in
	(compile) compile=yes;;
	(*) id=$opt;;
esac
done

source="$PWD"/UniversalMediaServer
if [ ! -e $source ] ; then
	git clone https://github.com/UniversalMediaServer/UniversalMediaServer.git
	cd $source
	git config --add remote.origin.fetch "+refs/pull/*/head:refs/remotes/origin/pr/*"
	cd ..
fi

cd $source
git stash
git checkout master
git pull

# select

if [ "$compile" = "" ] ; then
	menu "Select an option" 1 '"Build the full package" "Compile only"'
	case $index in
		(2) compile=yes;;
	esac
fi

if [ "$id" = "" ] ; then
	menu "Select an option" 1 '"Build the master branch" "Choose another branch" "Choose a pull request" "Show all" "Rebuild" "Specify a commit or tag"'
	case $index in
		(1) branch=master ;;
		(2) branches ;;
		(3) prs ;;
		(4) all ;;
		(5) existing ;;
		(6) specific ;;
	esac
	[ "$id" = "" ] && id=$branch
fi

[ "$branch" != "" ] && echo "branch=$branch"

# build

git checkout $id
[ $? != 0 ] && exit 1

if [ "$branch" != "" ] ; then
	if [[ "$branch" == *pr/* ]] && [[ "$(git config branch.$branch.merge)" == */pr/* ]]; then
		# for some reason 'refs/heads/pr/362' won't pull but 'refs/pull/362/head' will
		pr=${branch##pr/}
		git config branch.$branch.merge "refs/pull/$pr/head" 
	fi

	git pull --ff-only
	[ $? != 0 ] && exit 1
fi

pkg=distribution.tar.gz
ext=tgz
if [ "$id" = "" ] ; then
	id=$(git branch)
	id=${id##*\* }
	id=${id%% *}
fi
id=${id/ /_}
id=${id/\//}
id=$id-$(git rev-parse HEAD | tr -d '\n' | cut -c 1-7)

echo "id=$id"

if [ "$compile" = "yes" ] ; then
	mvn -DskipTests -Doffline compile
	exit
fi

# alternatively:
#if [ "$(awk '/mvn external:install/' pom.xml)" == "" ] ; then

if [ "$(grep -c 'mvn external:install' pom.xml)" == "0" ] ; then
	mvn com.savage7.maven.plugins:maven-external-dependency-plugin:resolve-external
	mvn com.savage7.maven.plugins:maven-external-dependency-plugin:install-external
else
	mvn external:install
fi

mvn clean
mvn -P linux package

# finish

mv target/ums-*-distribution.tar.gz ../ums-$id.tar.gz
mv target/ums.jar ../ums-$id.jar

[ -d keep ] && rm -fr keep
mkdir keep
mv -f target/bin keep/
mv -f target/maven-status keep/
mvn clean
mv keep target

if [ "$branch" = "" ] ; then git checkout master; fi

cd ..
echo
echo "Done: ums-$id.tar.gz"
echo
